@@ -14,13 +14,14 @@ from sksurgeryimage.calibration.point_detector import PointDetector
 
 LOGGER = logging.getLogger(__name__)
 
+
 def get_model_points(dots_rows_columns: (int, int),
                      pixels_per_mm: int,
                      dot_separation: float) -> np.ndarray:
     """Generate the expected locations of dots in the pattern, in pixel space.
 
     :param dots_rows_columns: Number of rows, number of columns
-    :type number_of_dots: [int, int]
+    :type dots_rows_columns: [int, int]
     :param pixels_per_mm: Pixels per mm
     :type pixels_per_mm: int
     :param dot_separation: Distance between dots in mm
@@ -44,6 +45,7 @@ def get_model_points(dots_rows_columns: (int, int),
 
     return model_points
 
+
 class DottyGridPointDetector(PointDetector):
     """
     Class to detect a grid of dots in a 2D grey scale video image.
@@ -53,7 +55,7 @@ class DottyGridPointDetector(PointDetector):
     def __init__(self,
                  model_points,
                  list_of_indexes,
-                 intrinsics,
+                 camera_intrinsics,
                  distortion_coefficients,
                  scale=(1, 1),
                  reference_image_size=None,
@@ -62,22 +64,24 @@ class DottyGridPointDetector(PointDetector):
                  threshold_window_size=151,
                  threshold_offset=20,
                  min_area=50,
-                 max_area=50000
+                 max_area=50000,
+                 dot_detector_params=None
                  ):
         """
         Constructs a PointDetector that extracts a grid of dots,
         with 4 extra large dots, at known locations.
 
-        Requires intrinsics and distortion_coefficients to be provided,
+        Requires camera_intrinsics and distortion_coefficients to be provided,
         then these are used as a reference transform to undistort
-        the image, which makes matching to a reference grid more reliable.
+        the image, which makes matching to a reference grid and identifying
+        point indexes more reliable.
 
         The list of indexes, must be of length 4, and correspond to
         top-left, top-right, bottom-left, bottom-right bigger blobs.
 
         :param model_points: numpy ndarray of id, x_pix, y_pix, x_mm, y_mm, z_mm
         :param list_of_indexes: list of specific indexes to use as fiducials
-        :param intrinsics: 3x3 ndarray of camera intrinsics
+        :param camera_intrinsics: 3x3 ndarray of camera intrinsics
         :param distortion_coefficients: 1x5 ndarray of distortion coeffs.
         :param scale: if you want to resize the image, specify scale factors
         :param reference_image_size: used to warp undistorted image to reference
@@ -87,15 +91,14 @@ class DottyGridPointDetector(PointDetector):
         :param threshold_offset: offset for adaptive thresholding
         :param min_area: minimum area when filtering by area
         :param max_area: maximum area when filtering by area
+        :param dot_detector_params: instance of cv2.SimpleBlobDetector_Params()
         """
-        super(DottyGridPointDetector, self).__init__(scale=scale)
+        super(DottyGridPointDetector, self).\
+            __init__(scale=scale,
+                     camera_intrinsics=camera_intrinsics,
+                     distortion_coefficients=distortion_coefficients
+                     )
 
-        if intrinsics is None:
-            raise ValueError('intrinsics is None')
-        if distortion_coefficients is None:
-            raise ValueError('distortion coefficients are None')
-        if len(distortion_coefficients) != 5:
-            raise ValueError('there should be 5 distortion coefficients')
         if len(list_of_indexes) != 4:
             raise ValueError('list_of_index not of length 4')
         if reference_image_size is None:
@@ -103,8 +106,6 @@ class DottyGridPointDetector(PointDetector):
 
         self.model_points = model_points
         self.list_of_indexes = list_of_indexes
-        self.intrinsics = intrinsics
-        self.distortion_coefficients = distortion_coefficients
         self.model_fiducials = self.model_points[self.list_of_indexes]
         self.reference_image_size = reference_image_size
         self.rms_tolerance = rms
@@ -114,6 +115,17 @@ class DottyGridPointDetector(PointDetector):
         self.min_area = min_area
         self.max_area = max_area
 
+        self.dot_detector_params = cv2.SimpleBlobDetector_Params()
+        self.dot_detector_params.filterByConvexity = False
+        self.dot_detector_params.filterByInertia = True
+        self.dot_detector_params.filterByCircularity = True
+        self.dot_detector_params.filterByArea = True
+        self.dot_detector_params.minArea = self.min_area
+        self.dot_detector_params.maxArea = self.max_area
+
+        if dot_detector_params is not None:
+            self.dot_detector_params = dot_detector_params
+
     def _internal_get_points(self, image, is_distorted=True):
         """
         Extracts points.
@@ -138,27 +150,18 @@ class DottyGridPointDetector(PointDetector):
                                             self.threshold_window_size,
                                             self.threshold_offset)
 
-        params = cv2.SimpleBlobDetector_Params()
-        params.filterByConvexity = False
-        params.filterByInertia = True
-        params.filterByCircularity = True
-        params.filterByArea = True
-        params.minArea = self.min_area
-        params.maxArea = self.max_area
-
         # Detect points in the distorted image
-        detector = cv2.SimpleBlobDetector_create(params)
+        detector = cv2.SimpleBlobDetector_create(self.dot_detector_params)
         keypoints = detector.detect(thresholded)
 
         # If input image is distorted, undistort and also detect points
         # in undistorted image.
         if is_distorted:
             undistorted_image = cv2.undistort(smoothed,
-                                              self.intrinsics,
+                                              self.camera_intrinsics,
                                               self.distortion_coefficients
                                               )
 
-
             undistorted_thresholded = \
                 cv2.adaptiveThreshold(undistorted_image,
                                       255,
@@ -171,7 +174,6 @@ class DottyGridPointDetector(PointDetector):
 
         else:
             undistorted_image = image
-            undistorted_thresholded = thresholded
             undistorted_keypoints = keypoints
 
         # Note that keypoints and undistorted_keypoints
@@ -326,51 +328,57 @@ class DottyGridPointDetector(PointDetector):
             for counter in range(len(inverted_points)):
                 matched_points[counter][0] = inverted_points[counter][0][0]
                 matched_points[counter][1] = inverted_points[counter][0][1]
+                img_points[counter][0] = inverted_points[counter][0][0]
+                img_points[counter][1] = inverted_points[counter][0][1]
 
-            # Now have to map undistorted points back to distorted points
-            for counter in range(number_of_warped_keypoints):
+            if is_distorted:
+                # Input image was a distorted image, so now we have to map
+                # undistorted points back to distorted points.
+                for counter in range(number_of_warped_keypoints):
+                    # Distort point to match original input image.
+                    relative_x = (matched_points[counter][0]
+                                  - self.camera_intrinsics[0][2]) \
+                                 / self.camera_intrinsics[0][0]
+                    relative_y = (matched_points[counter][1]
+                                  - self.camera_intrinsics[1][2]) \
+                                 / self.camera_intrinsics[1][1]
+                    r2 = relative_x * relative_x + relative_y * relative_y
+                    radial = (1
+                              + self.distortion_coefficients[0] * r2
+                              + self.distortion_coefficients[1] * r2 * r2
+                              + self.distortion_coefficients[4] * r2 * r2 * r2
+                              )
+                    distorted_x = relative_x * radial
+                    distorted_y = relative_y * radial
 
-                # Distort point to match original input image.
-                relative_x = (matched_points[counter][0]
-                              - self.intrinsics[0][2]) / self.intrinsics[0][0]
-                relative_y = (matched_points[counter][1]
-                              - self.intrinsics[1][2]) / self.intrinsics[1][1]
-                r2 = relative_x * relative_x + relative_y * relative_y
-                radial = (1
-                          + self.distortion_coefficients[0] * r2
-                          + self.distortion_coefficients[1] * r2 * r2
-                          + self.distortion_coefficients[4] * r2 * r2 * r2
-                          )
-                distorted_x = relative_x * radial
-                distorted_y = relative_y * radial
+                    distorted_x = distorted_x + (
+                            2 * self.distortion_coefficients[2]
+                            * relative_x * relative_y
+                            + self.distortion_coefficients[3]
+                            * (r2 + 2
+                               * relative_x
+                               * relative_x))
 
-                distorted_x = distorted_x + (2 * self.distortion_coefficients[2]
-                                             * relative_x * relative_y
-                                             + self.distortion_coefficients[3]
-                                             * (r2 + 2
-                                                * relative_x
-                                                * relative_x))
+                    distorted_y = distorted_y + (
+                            self.distortion_coefficients[2]
+                            * (r2 + 2 * relative_y
+                               * relative_y)
+                            + 2 *
+                            self.distortion_coefficients[3]
+                            * relative_x * relative_y)
 
-                distorted_y = distorted_y + (self.distortion_coefficients[2]
-                                             * (r2 + 2 * relative_y
-                                                * relative_y)
-                                             + 2 *
-                                             self.distortion_coefficients[3]
-                                             * relative_x * relative_y)
+                    distorted_x = distorted_x * self.camera_intrinsics[0][0] \
+                                  + self.camera_intrinsics[0][2]
+                    distorted_y = distorted_y * self.camera_intrinsics[1][1] \
+                                  + self.camera_intrinsics[1][2]
 
-                distorted_x = distorted_x * self.intrinsics[0][0] \
-                    + self.intrinsics[0][2]
-                distorted_y = distorted_y * self.intrinsics[1][1] \
-                    + self.intrinsics[1][2]
+                    img_points[counter][0] = distorted_x
+                    img_points[counter][1] = distorted_y
 
-                img_points[counter][0] = distorted_x
-                img_points[counter][1] = distorted_y
-
-            unique_ids, unique_idxs, counts = \
+            _, unique_idxs, counts = \
                 np.unique(indexes, return_index=True, return_counts=True)
 
-            unique_ids = unique_ids[counts==1]
-            unique_idxs = unique_idxs[counts==1]
+            unique_idxs = unique_idxs[counts == 1]
 
             indexes = indexes[unique_idxs]
             object_points = object_points[unique_idxs]
